/* 
   COPYRIGHT 2008 KRUPCZAK.ORG, LLC.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA
 
   For more information, visit:
   http://www.krupczak.org/
*/

/* file - xmptomrtg.c */
/* rdk@krupczak.org */

#ifdef WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#include <windows.h>
#include <pthread.h>
#endif

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#ifdef SOLARIS
#include <netinet/in.h>
#include <inttypes.h>
#endif

#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#include <strings.h>
#endif

#include <stdlib.h>

#include "xmp.h"
#include "xmp_session.h"
#include "socklib.h"

/* format of versionStr variable is "KorgVersion<space>%3.2f" 
   that way, our software distribution and updating component
   can correctly figure out version of some distributable
   binary thing */   

char *versionStr = "KorgVersion 1.01";
xmpSession_t *sptr = NULL;


/* added support for SSL tools */

/* declare these variables for xmp and xmp_session */

xmpStats_t xmpStats;
int xmp_session_verbose = 0;

int verbose = 0;

int get_fqdn(char *, int);
int getvars(char *,char *, char *, char *,
            char *,char *, char *, char *);
int get_target_uptime();

/* ************************************************************* */

int main(int argc, char *argv[])
{
     u_int target;
     short xmpPort = XMP_PORT;
     char *authenUser;
     char table1[128],table2[128];
     char mib1[128],mib2[128];
     char var1[128],var2[128];
     char instance1[256],instance2[256];
     char *cptr;
     char fqdn[512];

#ifdef WIN32
     WSADATA wsaData;

     if (WSAStartup(0x202,&wsaData) == SOCKET_ERROR) {
        perror("WSAStartup failed");
        WSACleanup();
        exit(-1);
     }
#endif

     if (argc < 5) {
        fprintf(stderr,"usage: %s host[:port] authenUser mib1[-table1]:var1[,instance1] "
            "mib2[-table2]:var2[,instance2]\n",argv[0]);
        exit(-1);
     }

     /* figure out our FQDN */
     get_fqdn(fqdn,511);

#if 0
     fprintf(stderr,"xmpquery: using fqdn '%s'\n",fqdn);
#endif

     init_ssl_stuff(fqdn,"xmptomrtg.pem","xmptomrtg.csr");

     /* now, connect to host/5270 */

     if ((cptr = strchr(argv[1],':')) != NULL) {
        *cptr = 0;
        cptr++;
        xmpPort = atoi(cptr);
     }

     target = hostname_to_ipaddr(argv[1]);

     authenUser = argv[2];

     sptr = newXmpSessionConnectTimeout(target,htons(xmpPort),3,authenUser);

     if (sptr == NULL) {
        fprintf(stderr,"xmptomrtg: failed to connect to %s\n",argv[1]);
        exit(-1);
     }

     if (verbose)
        fprintf(stderr,"xmptomrtg: connected to %s\n",argv[1]);

     /* parse the variables to query

        Examples:

             scalars:  cartographer:loadAvg1Min
             tabular:  cartographer-filesysTable:usedBlocks,/var
     */

     /* parse variables var1=argv[3] var2=argv[4] */
     /* work backwards */
 
     /* var1 */
     mib1[0] = table1[0] = var1[0] = instance1[0] = 0;
     /* instance */
     cptr = strrchr(argv[3],',');
     if (cptr != NULL) {
        *cptr = 0;
        cptr++;
        strcpy(instance1,cptr);
     }
     /* var */
     cptr = strrchr(argv[3],':');
     if (cptr != NULL) {
        *cptr = 0;
        cptr++;
        strcpy(var1,cptr);
     }
     /* table */
     cptr = strrchr(argv[3],'-');
     if (cptr != NULL) {
        *cptr = 0;
        cptr++;
        strcpy(table1,cptr);
     }
     strcpy(mib1,argv[3]);

     /* var2 */
     mib2[0] = table2[0] = var2[0] = instance2[0] = 0;
     /* instance */
     cptr = strrchr(argv[4],',');
     if (cptr != NULL) {
        *cptr = 0;
        cptr++;
        strcpy(instance2,cptr);
     }
     /* var */
     cptr = strrchr(argv[4],':');
     if (cptr != NULL) {
        *cptr = 0;
        cptr++;
        strcpy(var2,cptr);
     }
     /* table */
     cptr = strrchr(argv[4],'-');
     if (cptr != NULL) {
        *cptr = 0;
        cptr++;
        strcpy(table2,cptr);
     }
     strcpy(mib2,argv[4]);

     if (verbose) {
        fprintf(stderr,"Var1 mib:'%s' table:'%s' var:'%s' instance:'%s'\n",
                mib1,table1,var1,instance1);
        fprintf(stderr,"Var2 mib:'%s' table:'%s' var:'%s' instance:'%s'\n",
                mib2,table2,var2,instance2);
     }

     /* get values and do output according to specs */
     getvars(mib1,table1,var1,instance1,mib2,table2,var2,instance2);
     get_target_uptime();
     printf("%s\n",argv[1]);

     destroyXmpSession(sptr);

     exit(0);
}

/* ************************************************************* */
/* get and print uptime */

int get_target_uptime()
{
     xmpPdu_t query, reply;
     xmpVariable_t aVar1, *varptr;
     int copyLen,ret;
     char aStr[4096];

     /* build the PDU */
     bzero((void *)&query,sizeof(query));

     query.msgType = GetRequest;
     query.messageID = (int64_t)time(NULL);
     query.errorStatus = noError;
     query.errorIndex = 0;

     /* put vars in */
     query.numberVariables = 1;
     query.varList = &aVar1;
     query.varListTail = NULL;
     bzero((void *)&aVar1,sizeof(aVar1));
     aVar1.next = NULL;

     sprintf(aVar1.varName.mibName,"core");
     sprintf(aVar1.varName.objName,"sysUpTime");
     aVar1.varSyntax = nullSyntax;
     aVar1.valueStrLen = 0;

     bzero((void *)&reply,sizeof(reply));

     ret = xmpSendQueryReceiveReply(sptr,&query,&reply);

     if (ret < 0) {
        fprintf(stderr,"xmptomrtg: var query failed\n");
        exit(-1);
     }

     if ((reply.errorStatus == noError) &&
	 (reply.numberVariables == 1)) {

         varptr = reply.varList;

         copyLen = varptr->valueStrLen+1; /* + \0 */
         strncpy(aStr,varptr->valueStr,copyLen);
         aStr[copyLen-1] = 0;
         printf("XMP Daemon uptime %s\n",aStr);
     }

     return 1;
}

/* ************************************************************* */
/* if tabular, vars have to be in same table */

int getvars(char *mib1,char *table1, char *var1, char *instance1,
            char *mib2,char *table2, char *var2, char *instance2)
{
     xmpPdu_t query, reply;
     xmpVariable_t aVar1,aVar2, *varptr;
     int copyLen,ret;
     char aStr[4096];
     float f;
 
     /* some checks */
     if ((table1[0] == 0) && (table2[0] != 0)) {
        fprintf(stderr,"Tables do not match\n");
        return -1;
     }
     if ((table1[0] != 0) && (table2[0] == 0)) {
        fprintf(stderr,"Tables do not match\n");
        return -1;
     }
     if ((table1[0] != 0) && (table2[0] != 0)) {
        if (strcmp(table1,table2) != 0) {
           fprintf(stderr,"Tables do not match\n");
           return -1;
        }
     }
     if (table1[0] != 0) {
        if ((instance1[0] == 0) || (instance2[0] == 0)) {
           fprintf(stderr,"Must have instances if tabular data\n");
           return -1;
        }
        if (strcmp(instance1,instance2) != 0) {
           fprintf(stderr,"Instances do not match\n");
           return -1;
        }
     }

     /* build the PDU */
     bzero((void *)&query,sizeof(query));

     /* if tabular or scalar */
     if (table1[0] != 0) {
        query.msgType = SelectTableRequest;
        query.messageID = (int64_t)time(NULL);
        query.errorStatus = noError;
        query.errorIndex = 0;
        sprintf(query.tableMib,"%s",mib1);
        sprintf(query.tableName,"%s",table1);
        sprintf(query.tableKey,"%s",instance1);
     }
     else {
        query.msgType = GetRequest;
        query.messageID = (int64_t)time(NULL);
        query.errorStatus = noError;
        query.errorIndex = 0;
     }

     /* put vars in */
     query.numberVariables = 2;
     query.varList = &aVar1;
     query.varListTail = &aVar2;
     bzero((void *)&aVar1,sizeof(aVar1));
     bzero((void *)&aVar2,sizeof(aVar2));
     aVar1.next = &aVar2;

     sprintf(aVar1.varName.mibName,"%s",mib1);
     sprintf(aVar1.varName.objName,"%s",var1);
     aVar1.varSyntax = nullSyntax;
     aVar1.valueStrLen = 0;

     sprintf(aVar2.varName.mibName,"%s",mib2);
     sprintf(aVar2.varName.objName,"%s",var2);
     aVar2.varSyntax = nullSyntax;
     aVar2.valueStrLen = 0;

     bzero((void *)&reply,sizeof(reply));

     ret = xmpSendQueryReceiveReply(sptr,&query,&reply);

     if (ret < 0) {
        fprintf(stderr,"xmptomrtg: var query failed\n");
        exit(-1);
     }

     /* we should print out first two answers if its
        a table query since we might select multiple
        matches;  */

     /* also, if its verbose, print out all the responses
        instead of first two */

     if ((reply.errorStatus == noError) &&
	 (reply.numberVariables >= 2)) {

         varptr = reply.varList;

         copyLen = varptr->valueStrLen+1; /* + \0 */
         strncpy(aStr,varptr->valueStr,copyLen);
         aStr[copyLen-1] = 0;

         /* if float */
         if (varptr->varSyntax == floatingPoint) {
	    f = atof(aStr);
            f = f*100.0;
            sprintf(aStr,"%3.2f",f);
         }

         printf("%s\n",aStr);

         varptr = varptr->next;
         copyLen = varptr->valueStrLen+1; /* + \0 */
         strncpy(aStr,varptr->valueStr,copyLen);
         aStr[copyLen-1] = 0;

         /* if float */
         if (varptr->varSyntax == floatingPoint) {
	    f = atof(aStr);
            f = f*100.0;
            sprintf(aStr,"%3.2f",f);
         }

         printf("%s\n",aStr);

         /* if verbose */
         if (verbose) {
	    varptr = varptr->next;
            while (varptr != NULL) {

              copyLen = varptr->valueStrLen+1; /* + \0 */
              strncpy(aStr,varptr->valueStr,copyLen);
              aStr[copyLen-1] = 0;

              /* if float */
              if (varptr->varSyntax == floatingPoint) {
   	         f = atof(aStr);
                 f = f*100.0;
                 sprintf(aStr,"%3.2f",f);
              }
              printf("%s\n",aStr);

              varptr = varptr->next;

            } /* while verbose and more vars to print */

         } /* if verbose */

     } /* if reply OK */

     return 1;
}

/* ************************************************************* */
/* output of following format:

Line 1 current state of the first variable, normally 'incoming bytes count' 
Line 2 current state of the second variable, normally 'outgoing bytes count' 
Line 3 string (in human readable format), telling the uptime of the target. 
Line 4 string, telling the name of the target.

*/


/* ************************************************************* */

